<?php
class Vendorfuel_Templates {

	protected $utilities;

	public function __construct() {
		$this->utilities = Vendorfuel_Utilities::get_instance();

		if (!function_exists('WP_Filesystem')) {
			require_once ABSPATH . 'wp-admin/includes/file.php';
		}

		WP_Filesystem();

		//custom posts for SEO/Frontend deployment
		add_action('init', array($this, 'create_post_types'));
		add_action('the_post', array($this, 'vf_custom_post_types_remove_rss_feed'));

		//templates
		add_shortcode('vf-template', array($this, 'vendorfuel_template_shortcode'));
		add_action('permalink_structure_changed', array($this, 'permalink_template_map_update'));

		//yoast custom variable
		add_action('wpseo_register_extra_replacements', array($this, 'register_custom_yoast_variables'), 10, 0);

		//template map permalink updates
		add_action('save_post',  array($this,  'permalink_template_map_update'), 10, 1);

		//insert required helper component
		add_action(
			'wp_body_open',
			function () {
				echo '<vf-helper></vf-helper>';
			}
		);
	}

	private static $instance = null;

	public static function get_instance() {
		if (null === self::$instance) {
			self::$instance = new Vendorfuel_Templates();
		}
		return self::$instance;
	}



	public function shortcode($name) {
		return do_shortcode($this->utilities->get_template_content($name));
	}

	/**
	 * Processes vf template shortcode params and replaces content accordingly.
	 *
	 * @param array $params The params passed in from the shortcode
	 *
	 * @return string The html content of the template after params processed.
	 */
	public function vendorfuel_template_shortcode($params) {
		if (isset($params['name'])) {
			$content = $this->shortcode("{$params['name']}");

			if (isset($params['po_prefix'])) {
				return str_replace('PO_PREFIX', $params['po_prefix'], $content);
			}

			if (isset($params['product_id'])) {
				return str_replace('ID_OF_PRODUCT', $params['product_id'], $content);
			}

			if (isset($params['cat_id'])) {
				$content = str_replace('ID_OF_CATALOG', $params['cat_id'], $content);
				return str_replace('TYPE_OF_ID', 'CATEGORY', $content);
			}

			if (isset($params['collection_id'])) {
				$content = str_replace('ID_OF_CATALOG', $params['collection_id'], $content);
				return str_replace('TYPE_OF_ID', 'COLLECTION', $content);
			}

			if (isset($params['mfg_id'])) {
				$content = str_replace('ID_OF_CATALOG', $params['mfg_id'], $content);
				return str_replace('TYPE_OF_ID', 'MANUFACTURER', $content);
			}

			if (isset($params['customer_role'])) {
				return str_replace('CUSTOMER_ROLE', $params['customer_role'], $content);
			}

			return $content;
		}
		return '';
	}

	public function create_post_types() {
		$general_settings = get_option('vendorfuel-general-settings');
		register_post_type(
			'vf-product',
			array(
				'labels'          => array(
					'name'						=> __('VendorFuel Product Detail Pages'),
					'singular_name'				=> __('Product Detail Page'),
					'add_new_item'				=> __('Add New Product Detail Page'),
					'edit_item'					=> __('Edit Product Detail Page'),
					'new_item'					=> __('New Product Detail Page'),
					'view_item'					=> __('View Product Detail Page'),
					'view_items'				=> __('View Product Detail Pages'),
					'search_items'				=> __('Search Product Detail Pages'),
					'not_found'					=> __('No Product Detail Pages found.'),
					'not_found_in_trash'		=> __('No Product Detail Pages found in Trash.'),
					'all_items'					=> __('All Product Detail Pages'),
					'archives'					=> __('Product Detail Page Archives'),
					'attributes'				=> __('Product Detail Page Attributes'),
					'insert_into_item'			=> __('Insert into product detail page'),
					'uploaded_to_this_item'		=> __('Uploaded to this product detail page'),
					'menu_name'					=> __('VF Product Detail Pages'),
					'filter_items_list'			=> __('Filter product detail pages list'),
					'items_list_navigation'		=> __('Product detail pages list navigation'),
					'items_list'				=> __('Product detail pages list'),
					'item_published'			=> __('Product Detail Page published.'),
					'item_published_privately'	=> __('Product Detail Page published privately.'),
					'item_reverted_to_draft'	=> __('Product Detail Page reverted to draft.'),
					'item_scheduled'			=> __('Product Detail Page scheduled.'),
					'item_updated'				=> __('Product Detail Page updated.'),
					'item_link'					=> __('Product Detail Page Link.'),
					'item_link_description'		=> __('A link to a product detail page.'),
				),
				'public'          => true,
				'has_archive'     => false,
				'rewrite'         => array('slug' => (isset($general_settings->product_slug) && '' !== $general_settings->product_slug) ? $general_settings->product_slug : 'products'),
				'supports'        => array('editor', 'title', 'custom-fields'),
				'show_in_rest'    => true,
				'capability_type' => 'post',
			)
		);

		register_post_type(
			'vf-category',
			array(
				'labels'          => array(
					'name'						=> __('VendorFuel Category Pages'),
					'singular_name'				=> __('Category Page'),
					'add_new_item'				=> __('Add New Category Page'),
					'edit_item'					=> __('Edit Category Page'),
					'new_item'					=> __('New Category Page'),
					'view_item'					=> __('View Category Page'),
					'view_items'				=> __('View Category Pages'),
					'search_items'				=> __('Search Category Pages'),
					'not_found'					=> __('No Category Pages found.'),
					'not_found_in_trash'		=> __('No Category Pages found in Trash.'),
					'all_items'					=> __('All Category Pages'),
					'archives'					=> __('Category Page Archives'),
					'attributes'				=> __('Category Page Attributes'),
					'insert_into_item'			=> __('Insert into category page'),
					'uploaded_to_this_item'		=> __('Uploaded to this category page'),
					'menu_name'					=> __('VF Category Pages'),
					'filter_items_list'			=> __('Filter category pages list'),
					'items_list_navigation'		=> __('Category pages list navigation'),
					'items_list'				=> __('Category pages list'),
					'item_published'			=> __('Category Page published.'),
					'item_published_privately'	=> __('Category Page published privately.'),
					'item_reverted_to_draft'	=> __('Category Page reverted to draft.'),
					'item_scheduled'			=> __('Category Page scheduled.'),
					'item_updated'				=> __('Category Page updated.'),
					'item_link'					=> __('Category Page Link.'),
					'item_link_description'		=> __('A link to a category page.'),
				),
				'public'          => true,
				'has_archive'     => false,
				'rewrite'         => array('slug' => (isset($general_settings->cat_slug) && '' !== $general_settings->cat_slug) ? $general_settings->cat_slug : 'categories'),
				'supports'        => array('editor', 'title', 'custom-fields'),
				'show_in_rest'    => true,
				'capability_type' => 'post',
			)
		);
	}

	public function get_metadesc() {
		global $post;
		substr(get_post_meta($post->ID, 'vfmetadesc', true), 0, 155);
	}

	// define the action for register yoast_variable replacments
	public function register_custom_yoast_variables() {
		if (function_exists('wpseo_register_var_replacement')) {
			wpseo_register_var_replacement('%%vfmetadesc%%', array($this, 'get_metadesc'), 'advanced', 'VF Meta Description Field');
		}
	}

	public function vf_custom_post_types_remove_rss_feed() {
		if ('vf-product' === get_post_type() || 'vf-category' === get_post_type()) {
			remove_action('wp_head', 'feed_links_extra', 3);
			remove_action('wp_head', 'feed_links', 2);
		}
	}

	/**
	 * When permalinks setting is updated the template map urls will be updated.
	 *
	 */
	public function permalink_template_map_update() {
		//retrieve current anguledit settings where template_map is stored
		$anguledit_settings = get_option('vendorfuel-anguledit-settings');

		if (isset($anguledit_settings->template_map)) {
			foreach ($anguledit_settings->template_map as $slug => $template) {
				$template->url                             = get_permalink($template->id);
				$anguledit_settings->template_map->{$slug} = $template;
			}
			update_option('vendorfuel-anguledit-settings', $anguledit_settings);
		}
	}

	/**
	 * When a page/post is updated the template map will be searched for this post ID and url updated accordingly.
	 *
	 * @param integer $post_ID The ID of the post/page that was updated.
	 */
	public function update_template_map_links($post_ID) {
		//retrieve current anguledit settings where template_map is stored
		$anguledit_settings = get_option('vendorfuel-anguledit-settings');

		if (isset($anguledit_settings->template_map)) {
			//loop through template_map
			foreach ($anguledit_settings->template_map as $index => $template) {
				//match updated post ID with template_map post IDs
				if ($template->id === $post_ID) {
					//if matched
					//get new permalink of the updated post
					$template->url = get_permalink($post_ID);
					//store updated mapped template to template_map
					$anguledit_settings->template_map->{$index} = $template;
				}
			}
		}

		//persist updated template map
		update_option('vendorfuel-anguledit-settings', $anguledit_settings);
	}
}
