<?php
require_once 'class-vendorfuel-utilities.php';
require_once 'class-vendorfuel-enqueue.php';
require_once 'class-vendorfuel-admin-page.php';
require_once 'class-vendorfuel-initialize.php';
require_once 'class-vendorfuel-wprest-api.php';
require_once 'class-vendorfuel-templates.php';
require_once 'class-vendorfuel-queue.php';

/**
 * The core plugin class.
 *
 * @since 1.0.0
 *
 * @package    VendorFuel
 * @subpackage VendorFuel/includes
 */
class VendorFuel {

	private $templates;
	private $utilities;
	private $queue;

	public function __construct() {
		$this->templates = Vendorfuel_Templates::get_instance();
		$this->utilities = Vendorfuel_Utilities::get_instance();
		$this->queue     = Vendorfuel_Queue::get_instance();

		/**
		 * Admin assets
		 */
		add_action('admin_enqueue_scripts', array($this, 'vendorfuel_enqueue_plugin'));
		add_action('admin_init', array($this, 'vendorfuel_admin_init'));
		add_action('admin_menu', array($this, 'vendorfuel_admin_menu'));

		/**
		 * Frontend assets
		 */
		add_action('wp_enqueue_scripts', array($this, 'vendorfuel_enqueue_frontend'));

		add_action('rest_api_init', array($this, 'vendorfuel_custom_api'));
	}

	public function vendorfuel_enqueue_plugin($hook) {
		if ('toplevel_page_vendorfuel' === $hook) {
			$assets = new Vendorfuel_Enqueue($this->utilities);
			$assets->menu_page_assets($hook);
		} elseif (str_contains($hook, 'vendorfuel_page')) {
			$assets = new Vendorfuel_Enqueue($this->utilities);
			$assets->submenu_page_assets($hook);
		}
		return;
	}

	public function vendorfuel_enqueue_frontend($hook) {
		$assets = new Vendorfuel_Enqueue($this->utilities);
		$assets->frontend();
	}

	public function vendorfuel_admin_init($hook) {
		$vendorfuel = new Vendorfuel_Initialize($this->utilities);
		$vendorfuel->initialize();
	}

	public function vendorfuel_admin_menu($hook) {
		$menu = new Vendorfuel_Admin_Page();
		$menu->register_menu();
	}

	public function vendorfuel_custom_api($hook) {
		$rest_api = new Vendorfuel_WPRest_Api($this->utilities, $this->templates, $this->queue);
		$rest_api->register();
	}
}
